Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
UnionFind.HSTORESourceThe signature HSTORE describes an implementation of first-class *heterogeneous* stores, which can contain references of different types.
val new_store : unit -> storenew_store() creates an empty store.
copy s returns a copy of the store s. Every reference that is valid in the store s is also valid in the new store, and has the same content in both stores. The two stores are independent of one another: updating one of them does not affect the other. When supported, copy is cheap: it can be expected to run in constant time. However, some stores does not support copy; in that case, an unspecified exception is raised.
A reference of type 'a rref can be thought of as (a pointer to) an object that exists in some store.
make s v creates a fresh reference in the store s and sets its content to v. It updates the store in place and returns the newly-created reference.
get s x reads the current content of the reference x in the store s. It may update the store in place, and returns the current content of the reference.
set s x v updates the store s so as to set the content of the reference x to v. It updates the store in place.